@@ -22,6 +22,7 @@
 """
 
 
+
 import sys
 from inspect import currentframe, getouterframes
 import time
@@ -29,7 +30,7 @@ import os
 path = os.path
 import shutil
 
-from myhdl import _simulator, __version__
+from myhdl import _simulator, __version__, EnumItemType
 from myhdl._extractHierarchy import _HierExtr
 from myhdl import TraceSignalsError
 
@@ -45,10 +46,15 @@ _error.MultipleTraces = "Cannot trace multiple instances simultaneously"
 
 class _TraceSignalsClass(object):
 
-    __slot__ = ("name", )
+    __slot__ = ("name",
+                "timescale",
+                "tracelists"
+                )
 
     def __init__(self):
         self.name = None
+        self.timescale = "1ns"
+        self.tracelists = True
 
     def __call__(self, dut, *args, **kwargs):
         global _tracing
@@ -82,8 +88,8 @@ class _TraceSignalsClass(object):
             vcdfile = open(vcdpath, 'w')
             _simulator._tracing = 1
             _simulator._tf = vcdfile
-            _writeVcdHeader(vcdfile)
-            _writeVcdSigs(vcdfile, h.hierarchy)
+            _writeVcdHeader(vcdfile, self.timescale)
+            _writeVcdSigs(vcdfile, h.hierarchy, self.tracelists)
         finally:
             _tracing = 0
 
@@ -111,7 +117,7 @@ def _namecode(n):
         code = _codechars[r] + code
     return code
 
-def _writeVcdHeader(f):
+def _writeVcdHeader(f, timescale):
     print >> f, "$date"
     print >> f, "    %s" % time.asctime()
     print >> f, "$end"
@@ -119,11 +125,11 @@ def _writeVcdHeader(f):
     print >> f, "    MyHDL %s" % __version__
     print >> f, "$end"
     print >> f, "$timescale"
-    print >> f, "    1ns"
+    print >> f, "    %s" % timescale
     print >> f, "$end"
     print >> f
 
-def _writeVcdSigs(f, hierarchy):
+def _writeVcdSigs(f, hierarchy, tracelists):
     curlevel = 0
     namegen = _genNameCode()
     siglist = []
@@ -140,20 +146,43 @@ def _writeVcdSigs(f, hierarchy):
                 print >> f, "$upscope $end"
         print >> f, "$scope module %s $end" % name
         for n, s in sigdict.items():
-            if s._val == None:
+            if s._val is None:
                 raise ValueError("%s of module %s has no initial value" % (n, name))
             if not s._tracing:
                 s._tracing = 1
                 s._code = namegen.next()
                 siglist.append(s)
             w = s._nrbits
-            if w:
+            # use real for enum strings
+            if w and not isinstance(s._val, EnumItemType):
                 if w == 1:
                     print >> f, "$var reg 1 %s %s $end" % (s._code, n)
                 else:
                     print >> f, "$var reg %s %s %s $end" % (w, s._code, n)
             else:
                 print >> f, "$var real 1 %s %s $end" % (s._code, n)
+        # Memory dump by Frederik Teichert, http://teichert-ing.de, date: 2011.03.28
+        # The Value Change Dump standard doesn't support multidimensional arrays so 
+        # all memories are flattened and renamed.
+        if tracelists:
+            for n in memdict.keys():
+                memindex = 0
+                for s in memdict[n].mem:
+                    if s._val == None:
+                        raise ValueError("%s of module %s has no initial value" % (n, name))
+                    if not s._tracing:
+                        s._tracing = 1
+                        s._code = namegen.next()
+                        siglist.append(s)
+                    w = s._nrbits
+                    if w:
+                        if w == 1:
+                            print >> f, "$var reg 1 %s %s(%i) $end" % (s._code, n, memindex)
+                        else:
+                            print >> f, "$var reg %s %s %s(%i) $end" % (w, s._code, n, memindex)
+                    else:
+                        print >> f, "$var real 1 %s %s(%i) $end" % (s._code, n, memindex)
+                    memindex += 1
     for i in range(curlevel):
         print >> f, "$upscope $end"
     print >> f
