@@ -1,7 +1,7 @@
 #  This file is part of the myhdl library, a Python package for using
 #  Python as a Hardware Description Language.
 #
-#  Copyright (C) 2003-2009 Jan Decaluwe
+#  Copyright (C) 2003-2013 Jan Decaluwe
 #
 #  The myhdl library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public License as
@@ -21,26 +21,22 @@
 
 """
 
-
 import inspect
 # import compiler
 # from compiler import ast as astNode
-from types import GeneratorType, FunctionType, ClassType, MethodType
-from cStringIO import StringIO
+from types import FunctionType, MethodType
 import re
-import warnings
 import ast
 import __builtin__
 
 import myhdl
 from myhdl import *
 from myhdl import ConversionError
-from myhdl._unparse import _unparse
 from myhdl._cell_deref import _cell_deref
 from myhdl._always_comb import _AlwaysComb
+from myhdl._always_seq import _AlwaysSeq
 from myhdl._always import _Always
-from myhdl._delay import delay
-from myhdl.conversion._misc import (_error, _access, _kind, _context,
+from myhdl.conversion._misc import (_error, _access, _kind,
                                     _ConversionMixin, _Label, _genUniqueSuffix)
 from myhdl._extractHierarchy import _isMem, _getMemInfo, _UserCode
 from myhdl._Signal import _Signal, _WaiterList
@@ -51,6 +47,8 @@ myhdlObjects = myhdl.__dict__.values()
 builtinObjects = __builtin__.__dict__.values()
 
 _enumTypeSet = set()
+_constDict = {}
+_extConstDict = {}
 
 
 def _makeName(n, prefixes):
@@ -145,30 +143,39 @@ def _analyzeGens(top, absnames):
     for g in top:
         if isinstance(g, _UserCode):
             tree = g
-        elif isinstance(g, (_AlwaysComb, _Always)):
+        elif isinstance(g, (_AlwaysComb, _AlwaysSeq, _Always)):
             f = g.func
             s = inspect.getsource(f)
             s = _dedent(s)
             tree = ast.parse(s)
-            # print ast.dump(tree)
-            tree.sourcefile = inspect.getsourcefile(f)
+            #print ast.dump(tree)
+            tree.sourcefile  = inspect.getsourcefile(f)
             tree.lineoffset = inspect.getsourcelines(f)[1]-1
             tree.symdict = f.func_globals.copy()
             tree.callstack = []
             # handle free variables
+            tree.nonlocaldict = {}
             if f.func_code.co_freevars:
                 for n, c in zip(f.func_code.co_freevars, f.func_closure):
                     obj = _cell_deref(c)
                     if isinstance(g, _AlwaysComb):
-                        # print type(obj)
-                        assert isinstance(obj, (int, long, _Signal)) or \
-                               _isMem(obj) or _isTupleOfInts(obj)
+                        if not ( isinstance(obj, (int, long, EnumType,_Signal)) or \
+                                 _isMem(obj) or _isTupleOfInts(obj)
+                               ):
+                            info =  "File %s, line %s: " % (tree.sourcefile, tree.lineoffset)
+                            print type(obj)
+                            raise ConversionError(_error.UnsupportedType, n, info)
                     tree.symdict[n] = obj
+                    # currently, only intbv as automatic nonlocals (until Python 3.0)
+                    if isinstance(obj, intbv):
+                        tree.nonlocaldict[n] = obj
             tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
             v = _FirstPassVisitor(tree)
             v.visit(tree)
             if isinstance(g, _AlwaysComb):
                 v = _AnalyzeAlwaysCombVisitor(tree, g.senslist)
+            elif isinstance(g, _AlwaysSeq):
+                v = _AnalyzeAlwaysSeqVisitor(tree, g.senslist, g.reset, g.sigregs, g.varregs)
             else:
                 v = _AnalyzeAlwaysDecoVisitor(tree, g.senslist)
             v.visit(tree)
@@ -182,6 +189,7 @@ def _analyzeGens(top, absnames):
             tree.lineoffset = inspect.getsourcelines(f)[1]-1
             tree.symdict = f.f_globals.copy()
             tree.symdict.update(f.f_locals)
+            tree.nonlocaldict = {}
             tree.callstack = []
             tree.name = absnames.get(id(g), str(_Label("BLOCK"))).upper()
             v = _FirstPassVisitor(tree)
@@ -370,14 +378,6 @@ class _Rom(object):
     def __init__(self, rom):
         self.rom = rom
 
-
-def _maybeNegative(obj):
-    if hasattr(obj, '_min') and (obj._min is not None) and (obj._min < 0):
-        return True
-    if isinstance(obj, (int, long)) and obj < 0:
-        return True
-    return False
-
 re_str = re.compile(r"[^%]+")
 re_ConvSpec = re.compile(r"%(?P<justified>[-]?)(?P<width>[0-9]*)(?P<conv>[sd])")
 
@@ -429,13 +429,10 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         self.access = _access.INPUT
         self.kind = _kind.NORMAL
 
-
     def visit_BinOp(self, node):
         self.visit(node.left)
         self.visit(node.right)
         node.obj = int(-1)
-        node.signed = node.left.signed or node.right.signed
-       
 
     def visit_BoolOp(self, node):
         for n in node.values:
@@ -444,29 +441,31 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             if not hasType(n.obj, bool):
                 self.raiseError(node, _error.NotSupported, "non-boolean argument in logical operator")
         node.obj = bool()
-
- 
+    
     def visit_UnaryOp(self, node):
         self.visit(node.operand)
         op = node.op
         node.obj = node.operand.obj
-        node.signed = node.operand.signed
         if isinstance(op, ast.Not):
             node.obj = bool()
         elif isinstance(op, ast.UAdd):
             node.obj = int(-1)
         elif isinstance(op, ast.USub):
             node.obj = int(-1)
-            if isinstance(node.operand, ast.Num):
-                node.signed = True
-    
 
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setAttr(node)
         else:
             self.getAttr(node)
- 
+        if node.attr == 'next':
+            if isinstance(node.value, ast.Name):
+                n = node.value.id
+                obj = node.value.obj
+                if isinstance(obj, _Signal) and isinstance(obj._init, modbv):
+                    if not obj._init._hasFullRange():
+                        self.raiseError(node, _error.ModbvRange, n)
+         
     def setAttr(self, node):
         if node.attr != 'next':
             self.raiseError(node, _error.NotSupported, "attribute assignment")
@@ -478,7 +477,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
     def getAttr(self, node):
         self.visit(node.value)
         node.obj = None
-        node.signed = False
         if isinstance(node.value, ast.Name):
             n = node.value.id
             if (n not in self.tree.vardict) and (n not in self.tree.symdict):
@@ -507,9 +505,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 obj._setName(n+suf)
         if node.obj is None: # attribute lookup failed
             self.raiseError(node, _error.UnsupportedAttribute, node.attr)
-            
-
-
+        
     def visit_Assign(self, node):
         target, value = node.targets[0], node.value
         self.access = _access.OUTPUT
@@ -529,14 +525,21 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             if isinstance(obj, intbv):
                 if len(obj) == 0:
                     self.raiseError(node, _error.IntbvBitWidth, n)
-                    if obj._min < 0:
-                        _signed = True
+            if isinstance(obj, modbv):
+                if not obj._hasFullRange():
+                    self.raiseError(node, _error.ModbvRange, n)
+            ws = getattr(obj, 'lenStr', False)
+            ext = getattr(obj, 'external', False)
+	    if ws and ws in self.tree.symdict:
+                _constDict[ws] = self.tree.symdict[ws]
+                if ext:
+                    _extConstDict[ws] = self.tree.symdict[ws]
             if n in self.tree.vardict:
                 curObj = self.tree.vardict[n]
                 if isinstance(obj, type(curObj)):
                     pass
                 elif isinstance(curObj, type(obj)):
-                     self.tree.vardict[n] = obj
+                    self.tree.vardict[n] = obj
                 else:
                     self.raiseError(node, _error.TypeMismatch, n)
                 if getNrBits(obj) != getNrBits(curObj):
@@ -546,23 +549,15 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         else:
             self.visit(value)
 
-
-
-    
-
     def visit_AugAssign(self, node):
         self.access = _access.INOUT
         self.visit(node.target)
         self.access = _access.INPUT
         self.visit(node.value)
 
-
-
     def visit_Break(self, node):
         self.labelStack[-2].isActive = True
         
-
-
     def visit_Call(self, node):
         self.visit(node.func)
         self.access = _access.UNKNOWN
@@ -574,7 +569,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         argsAreInputs = True
         f = self.getObj(node.func)
         node.obj = None
-        node.signed = False
         if type(f) is type and issubclass(f, intbv):
             node.obj = self.getVal(node)
         elif f is concat:
@@ -592,7 +586,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         ### suprize: identity comparison on unbound methods doesn't work in python 2.5??
         elif f == intbv.signed:
             node.obj = int(-1)
-            node.signed = True
         elif f in myhdlObjects:
             pass
         elif f in builtinObjects:
@@ -609,6 +602,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             tree.sourcefile = inspect.getsourcefile(f)
             tree.lineoffset = inspect.getsourcelines(f)[1]-1
             tree.symdict = f.func_globals.copy()
+            tree.nonlocaldict = {}
             if fname in self.tree.callstack:
                 self.raiseError(node, _error.NotSupported, "Recursive call")
             tree.callstack = self.tree.callstack[:]
@@ -641,22 +635,16 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         elif type(f) is MethodType:
             self.raiseError(node,_error.NotSupported, "method call: '%s'" % f.__name__)
         else:
-            raise AssertionError("Unexpected callable")
+            debug_info = [e for e in ast.iter_fields(node.func)]
+            raise AssertionError("Unexpected callable %s" % str(debug_info))
         if argsAreInputs:
             for arg in node.args:
                 self.visit(arg)
 
-
-            
-
     def visit_Compare(self, node):
         node.obj = bool()
-        node.signed = False
-        #for n in ast.iter_child_nodes(node):
         for n in [node.left] + node.comparators:
             self.visit(n)
-            if n.signed:
-                node.signed = True
         op, arg = node.ops[0], node.comparators[0]
 ##         node.expr.target = self.getObj(arg)
 ##         arg.target = self.getObj(node.expr)
@@ -679,10 +667,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                     elif v == 1:
                         node.edge = sig.posedge
 
-
-
     def visit_Num(self, node):
-        node.signed = False
         n = node.n
         # assign to value attribute for backwards compatibility
         node.value = n
@@ -694,15 +679,11 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             node.obj = None
 
     def visit_Str(self, node):
-        node.signed = False
         node.obj = node.s
-
-            
+           
     def visit_Continue(self, node):
         self.labelStack[-1].isActive = True
 
-            
-
     def visit_For(self, node):
         node.breakLabel = _Label("BREAK")
         node.loopLabel = _Label("LOOP")
@@ -726,11 +707,9 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         self.labelStack.pop()
         self.labelStack.pop()
         
-
     def visit_FunctionDef(self, node):
         raise AssertionError("subclass must implement this")
 
-
     def visit_If(self, node):
         if node.ignore:
             return
@@ -759,7 +738,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             if not hasattr(test, 'case'):
                 return
             var, item = test.case
-            if var.obj != var1.obj or type(item) is not type(item1):
+            if var.id != var1.id or type(item) is not type(item1):
                 return
             if item in choices:
                 return
@@ -770,12 +749,16 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         if (len(choices) == _getNritems(var1.obj)) or node.else_:
             node.isFullCase = True
 
-
-
     def visit_ListComp(self, node):
         mem = node.obj = _Ram()
         self.kind = _kind.DECLARATION
-        self.visit(node.elt)
+        try:
+            self.visit(node.elt)
+        except ConversionError, e:
+            if e.kind == _error.UnboundLocal:
+                pass
+            else:
+                raise
         self.kind = _kind.NORMAL
         mem.elObj = self.getObj(node.elt)
         if not isinstance(mem.elObj, intbv) or not len(mem.elObj) > 0:
@@ -789,8 +772,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.UnsupportedListComp)
         mem.depth = cf.args[0].obj
 
-
-
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setName(node)
@@ -810,9 +791,9 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 obj = self.tree.vardict[n]
                 # upgrade bool to int for augmented assignments
                 if isinstance(obj, bool):
-                    obj = int(0)
+                    obj = int(-1)
                     self.tree.vardict[n] = obj
-            node.obj = obj
+                node.obj = obj
         else:
             if n in ("__verilog__", "__vhdl__"):
                     self.raiseError(node, _error.NotSupported,
@@ -824,9 +805,8 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
     def getName(self, node):
         n = node.id
         node.obj = None
-            
         if n not in self.refStack:
-            if n in self.tree.vardict:
+            if (n in self.tree.vardict) and (n not in self.tree.nonlocaldict):
                 self.raiseError(node, _error.UnboundLocal, n)
             self.globalRefs.add(n)
         if n in self.tree.sigdict:
@@ -840,6 +820,12 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             else:
                 if sig._type is bool:
                     node.edge = sig.posedge
+                ws = getattr(sig._val, 'lenStr', False)
+                ext = getattr(sig._val, 'external', False)
+	        if ws and ws in self.tree.symdict:
+                    _constDict[ws] = self.tree.symdict[ws]
+                    if ext:
+                        _extConstDict[ws] = self.tree.symdict[ws]
             if self.access == _access.INPUT:
                 self.tree.inputs.add(n)
             elif self.access == _access.OUTPUT:
@@ -856,7 +842,7 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             if self.access == _access.INOUT: # probably dead code
                 # upgrade bool to int for augmented assignments
                 if isinstance(obj, bool):
-                    obj = int(0)
+                    obj = int(-1)
                     self.tree.vardict[n] = obj
             node.obj = obj
         elif n in self.tree.symdict:
@@ -876,24 +862,34 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
                 else:
                     assert False, "unexpected mem access %s %s" % (n, self.access)
                 self.tree.hasLos = True
+                ws = getattr(m.elObj._val, 'lenStr', False)
+                ext = getattr(m.elObj._val, 'external', False)
+	        if ws and ws in self.tree.symdict:
+                    _constDict[ws] = self.tree.symdict[ws]
+                    if ext:
+                        _extConstDict[ws] = self.tree.symdict[ws]
             elif isinstance(node.obj, int):
                 node.value = node.obj
+                # put VHDL compliant integer constants in global dict
+                if n not in _constDict and abs(node.obj) < 2**31:
+                    _constDict[n] = node.obj
+            if n in self.tree.nonlocaldict:
+                # hack: put nonlocal intbv's in the vardict
+                self.tree.vardict[n] = v = node.obj
+                # typedef string for nonlocal intbv's
+                ws = getattr(v, 'lenStr', False)
+                ext = getattr(v, 'external', False)
+	        if ws and ws in self.tree.symdict:
+                    _constDict[ws] = self.tree.symdict[ws]
+                    if ext:
+                        _extConstDict[ws] = self.tree.symdict[ws]
         elif n in __builtin__.__dict__:
             node.obj = __builtin__.__dict__[n]
         else:
-            pass
-        node.signed = _maybeNegative(node.obj)
-##         node.target = node.obj
-
-
-
-
-
+            self.raiseError(node, _error.UnboundLocal, n)
         
     def visit_Return(self, node):
         self.raiseError(node, _error.NotSupported, "return statement")
-        
-
 
     def visit_Print(self, node):
         self.tree.hasPrint = True
@@ -950,8 +946,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.raiseError(node, _error.FormatString, "too many arguments")
         self.generic_visit(node)
         
-
-
     def visit_Subscript(self, node):
         if isinstance(node.slice, ast.Slice):
             self.accessSlice(node)
@@ -959,7 +953,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             self.accessIndex(node)
 
     def accessSlice(self, node):
-        node.signed = False
         self.visit(node.value)
         node.obj = self.getObj(node.value)
         self.access = _access.INPUT
@@ -995,15 +988,10 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
             node.obj = bool()
         else:
             node.obj = bool() # XXX default
-        node.signed = _maybeNegative(node.obj)
-
 
     def visit_Tuple(self, node):
-        node.signed = False
         self.generic_visit(node)
 
-
-
     def visit_While(self, node):
         node.breakLabel = _Label("BREAK")
         node.loopLabel = _Label("LOOP")
@@ -1027,7 +1015,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         self.labelStack.pop()
         self.labelStack.pop()
 
-
     def visit_Yield(self, node, *args):
         self.tree.hasYield += 1
         n = node.value
@@ -1047,8 +1034,6 @@ class _AnalyzeVisitor(ast.NodeVisitor, _ConversionMixin):
         node.senslist = senslist
 
         
-        
-
 class _AnalyzeBlockVisitor(_AnalyzeVisitor):
     
     def __init__(self, tree):
@@ -1139,7 +1124,25 @@ class _AnalyzeAlwaysCombVisitor(_AnalyzeBlockVisitor):
             for n in self.tree.outmems:
                 m = _getMemInfo(self.tree.symdict[n])
                 m._driven = "wire"
+                
+                
+class _AnalyzeAlwaysSeqVisitor(_AnalyzeBlockVisitor):
+    
+    def __init__(self, tree, senslist, reset, sigregs, varregs):
+        _AnalyzeBlockVisitor.__init__(self, tree)
+        self.tree.senslist = senslist
+        self.tree.reset = reset
+        self.tree.sigregs = sigregs
+        self.tree.varregs = varregs
 
+         
+    def visit_FunctionDef(self, node):
+        self.refStack.push()
+        for n in node.body:
+            self.visit(n)
+        self.tree.kind = _kind.ALWAYS_SEQ
+        self.refStack.pop()
+     
 
 class _AnalyzeAlwaysDecoVisitor(_AnalyzeBlockVisitor):
     
@@ -1156,7 +1159,6 @@ class _AnalyzeAlwaysDecoVisitor(_AnalyzeBlockVisitor):
         self.refStack.pop()
 
             
-            
 
 class _AnalyzeFuncVisitor(_AnalyzeVisitor):
     
@@ -1229,28 +1231,37 @@ class _AnalyzeFuncVisitor(_AnalyzeVisitor):
             self.tree.hasReturn = True
 
 
+ismethod = inspect.ismethod
+# inspect doc is wrong: ismethod checks both bound and unbound methods
+def isboundmethod(m):
+    return ismethod(m) and m.__self__ is not None
 
-       
 def _analyzeTopFunc(func, *args, **kwargs):
     tree = _makeAST(func)
-    v = _AnalyzeTopFuncVisitor(tree, *args, **kwargs)
+    v = _AnalyzeTopFuncVisitor(func, tree, *args, **kwargs)
     v.visit(tree)
     return v
-      
-    
+
 class _AnalyzeTopFuncVisitor(_AnalyzeVisitor):
 
-    def __init__(self, tree, *args, **kwargs):
+    def __init__(self, func, tree, *args, **kwargs):
+        self.func = func
         self.tree = tree
         self.args = args
         self.kwargs = kwargs
         self.name = None
         self.argdict = {}
-    
 
     def visit_FunctionDef(self, node):
+        
         self.name = node.name
         argnames = [arg.id for arg in node.args.args]
+        if isboundmethod(self.func):
+            if not argnames[0] == 'self':
+                self.raiseError(node, _error.NotSupported,
+                                "first method argument name other than 'self'")
+            # skip self
+            argnames = argnames[1:]  
         i=-1
         for i, arg in enumerate(self.args):
             n = argnames[i]
@@ -1265,8 +1276,4 @@ class _AnalyzeTopFuncVisitor(_AnalyzeVisitor):
                     self.argdict[n] = arg
                 if _isMem(arg):
                     self.raiseError(node, _error.ListAsPort, n)
-
         self.argnames = [n for n in argnames if n in self.argdict]
-
-
-        
\ No newline at end of file
