@@ -1,7 +1,7 @@
 #  This file is part of the myhdl library, a Python package for using
 #  Python as a Hardware Description Language.
 #
-#  Copyright (C) 2003-2008 Jan Decaluwe
+#  Copyright (C) 2003-2012 Jan Decaluwe
 #
 #  The myhdl library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public License as
@@ -23,19 +23,14 @@
 
 
 import sys
-import os
 import math
-import traceback
 import inspect
 from datetime import datetime
-import compiler
-# from compiler import ast as astNode
 import ast
 import string
 
-from types import GeneratorType, FunctionType, ClassType, TypeType, StringType
+from types import GeneratorType, ClassType, TypeType
 from cStringIO import StringIO
-import __builtin__
 import warnings
 
 import myhdl
@@ -44,10 +39,8 @@ from myhdl import ToVerilogError, ToVerilogWarning
 from myhdl._extractHierarchy import (_HierExtr, _isMem, _getMemInfo,
                                      _UserVerilogCode, _userCodeMap)
 
-from myhdl._always_comb import _AlwaysComb
-from myhdl._always import _Always
 from myhdl._instance import _Instantiator
-from myhdl.conversion._misc import (_error, _access, _kind, _context, 
+from myhdl.conversion._misc import (_error, _kind, _context, 
                                     _ConversionMixin, _Label, _genUniqueSuffix, _isConstant)
 from myhdl.conversion._analyze import (_analyzeSigs, _analyzeGens, _analyzeTopFunc, 
                                        _Ram, _Rom)
@@ -140,9 +133,12 @@ class _ToVerilogConvertor(object):
         # print h.top
         _checkArgs(arglist)
         genlist = _analyzeGens(arglist, h.absnames)
+        _annotateTypes(genlist)
         intf = _analyzeTopFunc(func, *args, **kwargs)
         intf.name = name
         doc = _makeDoc(inspect.getdoc(func))
+        
+        self._convert_filter(h, intf, siglist, memlist, genlist)
 
         _writeFileHeader(vfile, vpath, self.timescale)
         _writeModuleHeader(vfile, intf, doc)
@@ -159,6 +155,12 @@ class _ToVerilogConvertor(object):
             _writeTestBench(tbfile, intf)
             tbfile.close()
 
+        ### clean-up properly ###
+        self._cleanup(siglist)
+        
+        return h.top
+        
+    def _cleanup(self, siglist):
         # clean up signal names
         for sig in siglist:
             sig._clear()
@@ -175,7 +177,11 @@ class _ToVerilogConvertor(object):
         self.no_myhdl_header = False
         self.no_testbench = False
         
-        return h.top
+        
+    def _convert_filter(self, h, intf, siglist, memlist, genlist):
+        # intended to be a entry point for other uses: 
+        #  code checking, optimizations, etc
+        pass
     
 
 toVerilog = _ToVerilogConvertor()
@@ -288,7 +294,11 @@ def _writeSigDecls(f, intf, siglist, memlist):
         print >> f, "%s %s%s%s [0:%s-1];" % (k, p, r, m.name, m.depth)
     print >> f
     for s in constwires:
-        print >> f, "assign %s = %s;" % (s._name, int(s._val))
+        if s._type in (bool, intbv):
+            c = int(s.val)
+        else:
+            raise ToVerilogError("Unexpected type for constant signal", s._name)
+        print >> f, "assign %s = %s;" % (s._name, c)
     print >> f
     # shadow signal assignments
     for s in siglist:
@@ -367,6 +377,8 @@ def _convertGens(genlist, vfile):
             Visitor = _ConvertSimpleAlwaysCombVisitor
         elif tree.kind == _kind.ALWAYS_DECO:
             Visitor = _ConvertAlwaysDecoVisitor
+        elif tree.kind == _kind.ALWAYS_SEQ:
+            Visitor = _ConvertAlwaysSeqVisitor    
         else: # ALWAYS_COMB
             Visitor = _ConvertAlwaysCombVisitor
         v = Visitor(tree, blockBuf, funcBuf)
@@ -445,16 +457,20 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         # write size for large integers (beyond 32 bits signed)
         # with some safety margin
         # XXX signed indication 's' ???
+        p = abs(n)
         size = ''
-        num = str(n)
-        if radix == "hex":
+        num = str(p).rstrip('L')
+        if radix == "hex" or p >= 2**30:
             radix = "'h"
-            num = hex(n)[2:]
-        if n >= 2**30:
-            size = int(math.ceil(math.log(n+1,2))) + 1  # sign bit!
-            if not radix:
-                radix = "'d"
-        return "%s%s%s" % (size, radix, num)
+            num = hex(p)[2:].rstrip('L')
+        if p >= 2**30:
+            size = int(math.ceil(math.log(p+1,2))) + 1  # sign bit!
+#            if not radix:
+#                radix = "'d"
+        r = "%s%s%s" % (size, radix, num)
+        if n < 0: # add brackets and sign on negative numbers
+            r = "(-%s)" % r
+        return r
 
     def writeDeclaration(self, obj, name, dir):
         if dir: dir = dir + ' '
@@ -487,7 +503,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         else:
             self.write(";")
         
-
     def writeDeclarations(self):
         for name, obj in self.tree.vardict.items():
             self.writeline()
@@ -512,8 +527,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.write(senslist[-1]._toVerilog())
         self.write(")")
 
-
-
     def visit_BinOp(self, node):
         if isinstance(node.op, ast.Mod) and self.context == _context.PRINT:
             self.visit(node.left)
@@ -537,9 +550,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(")")
             self.context = None
 
-
-
-
     def checkOpWithNegIntbv(self, node, op):
         if op in ("+", "-", "*", "~", "&&", "||", "!"):
             return
@@ -549,9 +559,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 self.raiseError(node, _error.NotSupported,
                                 "negative intbv with operator %s" % op)
 
-
-
-
     def visit_BoolOp(self, node):
         self.write("(")
         self.visit(node.values[0])
@@ -560,14 +567,11 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.visit(n)
         self.write(")")
         
-
     def visit_UnaryOp(self, node):
         self.write("(%s" % opmap[type(node.op)])
         self.visit(node.operand)
         self.write(")")
 
-
-
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
             self.setAttr(node)
@@ -606,8 +610,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             e = getattr(obj, node.attr)
             self.write(e._toVerilog())
 
-
-
     def visit_Assert(self, node):
         self.write("if (")
         self.visit(node.test)
@@ -621,8 +623,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.writeline()
         self.write("end")
 
-
-
     def visit_Assign(self, node):
         # shortcut for expansion of ROM in case statement
         if isinstance(node.value, ast.Subscript) and \
@@ -653,6 +653,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.writeline()
             self.write("endcase")
             return
+        elif isinstance(node.value, ast.ListComp):
+            # skip list comprehension assigns for now
+            return
         # default behavior
         self.visit(node.targets[0])
         if self.isSigAss:
@@ -663,8 +666,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.visit(node.value)
         self.write(';')
 
-
-
     def visit_AugAssign(self, node, *args):
         # XXX apparently no signed context required for augmented assigns
         self.visit(node.target)
@@ -674,12 +675,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.visit(node.value)
         self.write(";")
 
-
-
     def visit_Break(self, node,):
         self.write("disable %s;" % self.labelStack[-2])
 
-
     def visit_Call(self, node):
         self.context = None
         fn = node.func
@@ -712,7 +710,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             # convert number argument to integer
             if isinstance(node.args[0], ast.Num):
                 node.args[0].n = int(node.args[0].n)
-        elif f is intbv:
+        elif f in (intbv, modbv):
             self.visit(node.args[0])
             return
         elif f == intbv.signed: # note equality comparison
@@ -752,7 +750,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             v = Visitor(node.tree, self.funcBuf)
             v.visit(node.tree)
 
-
     def visit_Compare(self, node):
         self.context = None
         if node.signed:
@@ -764,8 +761,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.write(")")
         self.context = None
 
-
-
     def visit_Num(self, node):
         if self.context == _context.PRINT:
             self.write('"%s"' % node.n)
@@ -781,12 +776,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         else:
             self.write(s)
 
-
-
     def visit_Continue(self, node):
         self.write("disable %s;" % self.labelStack[-1])
 
-
     def visit_Expr(self, node):
         expr = node.value
         # docstrings on unofficial places
@@ -801,8 +793,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         # ugly hack to detect an orphan "task" call
         if isinstance(expr, ast.Call) and hasattr(expr, 'tree'):
             self.write(';')
-            
-            
+                       
     def visit_IfExp(self, node):
         self.visit(node.test)
         self.write(' ? ')
@@ -810,7 +801,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.write(' : ')
         self.visit(node.orelse)
 
-
     def visit_For(self, node):
         self.labelStack.append(node.breakLabel)
         self.labelStack.append(node.loopLabel)
@@ -873,11 +863,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.labelStack.pop()
         self.labelStack.pop()
 
-
     def visit_FunctionDef(self, node):
         raise AssertionError("To be implemented in subclass")
 
-
     def visit_If(self, node):
         if node.ignore:
             return
@@ -951,14 +939,15 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.writeline()
             self.write("end")
 
-
     def visitKeyword(self, node, *args):
         self.visit(node.expr)
 
     def visit_Module(self, node, *args):
         for stmt in node.body:
             self.visit(stmt)
-       
+            
+    def visit_ListComp(self, node):
+        pass # do nothing
 
     def visit_Name(self, node):
         if isinstance(node.ctx, ast.Store):
@@ -1013,11 +1002,9 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         if addSignBit:
             self.write("})")       
 
-
     def visit_Pass(self, node):
         self.write("// pass")
 
-    
     def visit_Print(self, node):
         argnr = 0
         for s in node.format:
@@ -1064,17 +1051,13 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
                 self.context = _context.UNKNOWN
             self.writeline()
         self.write('$write("\\n");')
-        
-
 
     def visit_Raise(self, node):
         self.write("$finish;")
-
     
     def visit_Return(self, node):
         self.write("disable %s;" % self.returnLabel)
 
-
     def visit_Subscript(self, node):
         if isinstance(node.slice, ast.Slice):
             self.accessSlice(node)
@@ -1083,7 +1066,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
 
     def accessSlice(self, node):
         if isinstance(node.value, ast.Call) and \
-           node.value.func.obj is intbv and \
+           node.value.func.obj in (intbv, modbv) and \
            _isConstant(node.value.args[0], self.tree.symdict):
             c = self.getVal(node)
             self.write("%s'h" % c._nrbits)
@@ -1112,7 +1095,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         if addSignBit:
             self.write("})")
 
-
     def accessIndex(self, node):
         addSignBit = isinstance(node.ctx, ast.Load) and \
                      (not node.signed) and \
@@ -1128,7 +1110,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         if addSignBit:
             self.write("})")
 
-
     def visit_stmt(self, body):
         for stmt in body:
             self.writeline()
@@ -1137,7 +1118,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             if isinstance(stmt, ast.Call) and hasattr(stmt, 'tree'):
                 self.write(';')
 
-
     def visit_Tuple(self, node):
         assert self.context != None
         sep = ", "
@@ -1147,8 +1127,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(sep)
             self.visit(elt)
 
-
-
     def visit_While(self, node):
         self.labelStack.append(node.breakLabel)
         self.labelStack.append(node.loopLabel)
@@ -1171,8 +1149,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         self.labelStack.pop()
         self.labelStack.pop()
 
-
-
     def visit_Yield(self, node):
         yieldObj = self.getObj(node.value)
         assert node.senslist
@@ -1187,10 +1163,6 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.writeSensitivityList(senslist)
             self.write(";")
 
-
-
-
-
         
 class _ConvertAlwaysVisitor(_ConvertVisitor):
     
@@ -1236,7 +1208,6 @@ class _ConvertInitialVisitor(_ConvertVisitor):
 
 
 
-
 class _ConvertAlwaysCombVisitor(_ConvertVisitor):
     
     def __init__(self, tree, blockBuf, funcBuf):
@@ -1245,7 +1216,6 @@ class _ConvertAlwaysCombVisitor(_ConvertVisitor):
             self.okSigAss = False
         self.funcBuf = funcBuf
 
-
     def visit_FunctionDef(self, node):
         self.writeDoc(node)
         self.writeAlwaysHeader()
@@ -1257,15 +1227,12 @@ class _ConvertAlwaysCombVisitor(_ConvertVisitor):
         self.writeline(2)
 
 
-
         
 class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
     
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
-
-
     def visit_Attribute(self, node):
         if isinstance(node.ctx, ast.Store):
             self.write("assign ")
@@ -1273,24 +1240,18 @@ class _ConvertSimpleAlwaysCombVisitor(_ConvertVisitor):
         else:
             self.getAttr(node)
 
-
     def visit_FunctionDef(self, node):
         self.writeDoc(node)
         self.visit_stmt(node.body)
         self.writeline(2)
 
-
-
-
-
-        
+     
 class _ConvertAlwaysDecoVisitor(_ConvertVisitor):
     
     def __init__(self, tree, blockBuf, funcBuf):
         _ConvertVisitor.__init__(self, tree, blockBuf)
         self.funcBuf = funcBuf
 
-
     def visit_FunctionDef(self, node):
         self.writeDoc(node)
         self.writeAlwaysHeader()
@@ -1302,7 +1263,62 @@ class _ConvertAlwaysDecoVisitor(_ConvertVisitor):
         self.writeline(2)
 
 
-       
+def _convertInitVal(reg, init):
+    if isinstance(reg, _Signal):
+        tipe = reg._type
+    else:
+        assert isinstance(reg, intbv)
+        tipe = intbv
+    if tipe is bool:
+        v = '1' if init else '0'
+    elif tipe is intbv:
+        v = "%s" % init
+    else:
+        assert isinstance(init, EnumItemType)
+        v = init._toVerilog()
+    return v
+ 
+
+class _ConvertAlwaysSeqVisitor(_ConvertVisitor):
+    
+    def __init__(self, tree, blockBuf, funcBuf):
+        _ConvertVisitor.__init__(self, tree, blockBuf)
+        self.funcBuf = funcBuf
+
+    def visit_FunctionDef(self, node):
+        self.writeDoc(node)
+        self.writeAlwaysHeader()
+        self.writeDeclarations()
+        reset = self.tree.reset
+        sigregs = self.tree.sigregs
+        varregs = self.tree.varregs
+        if reset is not None:
+            self.writeline()
+            self.write("if (%s == %s) begin" % (reset, int(reset.active)))
+            self.indent()
+            for s in sigregs:
+                self.writeline()
+                self.write("%s <= %s;" % (s, _convertInitVal(s, s._init)))
+            for v in varregs:
+                n, reg, init = v
+                self.writeline()
+                self.write("%s = %s;" % (n, _convertInitVal(reg, init)))
+            self.dedent()
+            self.writeline()
+            self.write("end")
+            self.writeline()
+            self.write("else begin")
+            self.indent()
+        self.visit_stmt(node.body)
+        self.dedent()
+        if reset is not None:
+            self.writeline()
+            self.write("end")
+            self.dedent()
+        self.writeline()
+        self.write("end")
+        self.writeline(2)
+
     
 class _ConvertFunctionVisitor(_ConvertVisitor):
     
@@ -1321,7 +1337,6 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
             self.writeline()
             self.writeDeclaration(obj, name, "input")
             
-
     def visit_FunctionDef(self, node):
         self.write("function ")
         self.writeOutputDeclaration()
@@ -1340,7 +1355,6 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
         self.write("endfunction")
         self.writeline(2)
 
-
     def visit_Return(self, node):
         self.write("%s = " % self.tree.name)
         self.visit(node.value)
@@ -1349,7 +1363,7 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
         self.write("disable %s;" % self.returnLabel)
 
 
-    
+
     
 class _ConvertTaskVisitor(_ConvertVisitor):
     
@@ -1367,7 +1381,6 @@ class _ConvertTaskVisitor(_ConvertVisitor):
             self.writeline()
             self.writeDeclaration(obj, name, dir)
             
-
     def visit_FunctionDef(self, node):
         self.write("task %s;" % self.tree.name)
         self.indent()
@@ -1384,3 +1397,125 @@ class _ConvertTaskVisitor(_ConvertVisitor):
         self.writeline()
         self.write("endtask")
         self.writeline(2)
+
+
+def _maybeNegative(obj):
+    if hasattr(obj, '_min') and (obj._min is not None) and (obj._min < 0):
+        return True
+    if isinstance(obj, (int, long)) and obj < 0:
+        return True
+    return False
+
+class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
+
+    def __init__(self, tree):
+        self.tree = tree
+        
+    def visit_FunctionDef(self, node):
+        # don't visit arguments and decorators
+        for stmt in node.body:
+            self.visit(stmt)   
+        
+    def visit_BinOp(self, node):
+        self.visit(node.left)
+        self.visit(node.right)
+        node.signed = node.left.signed or node.right.signed
+      
+    def visit_BoolOp(self, node):
+        for n in node.values:
+            self.visit(n)
+        node.signed = False
+
+    def visit_UnaryOp(self, node):
+        self.visit(node.operand)
+        node.signed = node.operand.signed
+        if isinstance(node.op, ast.USub):
+            node.obj = int(-1)
+            if isinstance(node.operand, ast.Num):
+                node.signed = True
+    
+    def visit_Attribute(self, node):
+        if isinstance(node.ctx, ast.Store):
+            self.setAttr(node)
+        else:
+            self.getAttr(node)
+         
+    def setAttr(self, node):
+        self.visit(node.value)
+
+    def getAttr(self, node):
+        node.signed = False
+        self.visit(node.value)
+                  
+    def visit_Call(self, node):
+        self.generic_visit(node)
+        f = self.getObj(node.func)
+        node.signed = False
+        ### suprize: identity comparison on unbound methods doesn't work in python 2.5??
+        if f == intbv.signed:
+            node.signed = True
+        elif hasattr(node, 'tree'):
+            v = _AnnotateTypesVisitor(node.tree)
+            v.visit(node.tree)
+            node.signed = _maybeNegative(node.tree.returnObj)  
+
+    def visit_Compare(self, node):
+        node.signed = False
+        #for n in ast.iter_child_nodes(node):
+        for n in [node.left] + node.comparators:
+            self.visit(n)
+            if n.signed:
+                node.signed = True
+                
+    def visit_If(self, node):
+        if node.ignore:
+            return
+        self.generic_visit(node)
+
+
+    def visit_Num(self, node):
+        node.signed = False
+
+    def visit_Str(self, node):
+        node.signed = False
+
+    def visit_Name(self, node):
+        if isinstance(node.ctx, ast.Store):
+            self.setName(node)
+        else:
+            self.getName(node)
+
+    def setName(self, node):
+        pass
+
+    def getName(self, node):
+        node.signed = _maybeNegative(node.obj)
+
+    def visit_Subscript(self, node):
+        if isinstance(node.slice, ast.Slice):
+            self.accessSlice(node)
+        else:
+            self.accessIndex(node)
+
+    def accessSlice(self, node):
+        node.signed = False
+        self.generic_visit(node)
+
+    def accessIndex(self, node):
+        node.signed = _maybeNegative(node.obj)
+        self.generic_visit(node)
+
+    def visit_Tuple(self, node):
+        node.signed = False
+        self.generic_visit(node)
+
+        
+def _annotateTypes(genlist):
+    for tree in genlist:
+        if isinstance(tree, _UserVerilogCode):
+            continue
+        v = _AnnotateTypesVisitor(tree)
+        v.visit(tree)
+        
+
+
